/* KDE 1.* -> KDE 2.* config file converter...           *
 * (c) 2000 Red Hat, Inc., Programmed by bero@redhat.com *
 * GPL                                                   */

#include "Files.h"
#include <qfile.h>
#include <pwd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <stdio.h>
#include <unistd.h>
#include <dirent.h>
#include <string.h>
#include <fcntl.h>
#include <stdlib.h>
#include <alloca.h>
#include <errno.h>
#include <iostream>
#include <ctype.h>
#include <qapplication.h>
#include <qmessagebox.h>
#include <qregexp.h>
#include <libintl.h>
#include <locale.h>

bool mkdir_p(QString const &dir, mode_t mode = 755)
{
	int loc=1;
	bool success=true;
	while((loc=dir.find('/', loc))>=0) {
		if(mkdir(QFile::encodeName(dir.left(loc)), mode) && (errno != EEXIST))
			success=false;
		loc++;
	}
	if(mkdir(QFile::encodeName(dir), mode))
	success=false;
	return success;
}


char *replace(char *s, char const *what, char const *with)
{
	char *result;
	char *loc;
	int i;
	int size=strlen(s);
	if(strlen(with)>strlen(what))
		size+=(strlen(with)-strlen(what))*strlen(s);
	result=(char *) malloc(size);
	memset(result, 0, size);
	loc=result;
	for(i=0; i<strlen(s); i++) {
		if(!strncmp(s+i, what, strlen(what))) {
			strcat(loc, with); loc+=strlen(with); i+=strlen(what)-1;
		} else
			*loc++=s[i];
	}
	result=(char *) realloc(result, strlen(result)+1);
	return result;
}

/* Upgrade KDE 1.x .kdelnk files to KDE 2.x files */
int updateDesktop(QString const &dir, bool isDesktop=false)
{
	QStringList subdirs=Files::glob(dir + "/*", Files::Dir);
	for(QStringList::ConstIterator it=subdirs.begin(); it!=subdirs.end(); it++) {
		if((*it).right(2)!="/." && (*it).right(3)!="/..")
			updateDesktop(*it, isDesktop);
	}
	QStringList OldFiles=Files::glob(dir + "/*.kdelnk", (Files::Types)(Files::File | Files::Link));
	for(QStringList::ConstIterator it=OldFiles.begin(); it!=OldFiles.end(); it++) {
		QString oldFile=*it;
		QString newFile=oldFile;
		newFile.replace(QRegExp("\\.kdelnk"), isDesktop ? "" : ".desktop");
		if(QFile::exists(newFile)) // A KDE2 file already exists - skip
			continue;
		FILE *in=fopen(QFile::encodeName(oldFile), "r");
		if(!in)
			continue;
		FILE *out=fopen(QFile::encodeName(newFile), "w");
		if(!out) {
			fclose(in);
			continue;
		}
		char buf[2048];
		while(fgets(buf, 2048, in) && !feof(in)) {
			if(!strncasecmp(buf, "# KDE Config", 12))
				continue;
			else if(!strncasecmp(buf, "[KDE Desktop Entry]", 19))
				strcpy(buf, "[Desktop Entry]\n\0");
			else if(!strncasecmp(buf, "MountPoint=\n", 12)) // KDE 1.x braindamage
				continue;
			else if(!strncasecmp(buf, "Icon=", 5) || !strncasecmp(buf, "UnmountIcon=", 12)) {
				// Strip .xpm, .png, .gif, ... unless this is a full path name
				if(!strchr(buf, '/') && buf[strlen(buf)-5]=='.') {
					buf[strlen(buf)-5]='\n';
					buf[strlen(buf)-4]=0;
				}
				if(!strcasecmp(buf, "Icon=printer\n"))
					strcpy(buf, "Icon=kljettool\n");
			}
			fputs(buf, out);
		}
		fclose(in);
		fclose(out);
		unlink(QFile::encodeName(oldFile));
	}
	return 0;
}

/* Upgrade a kpanelrc to a kickerrc, unless kickerrc already exists */
int updatePanel(QString const &homedir)
{
	QString old=homedir + "/.kde/share/config/kpanelrc";
	QString nw=homedir + "/.kde/share/config/kickerrc";
	if(!QFile::exists(old) || QFile::exists(nw))
		return 0;

	int found=0;
	FILE *in=NULL, *out=NULL;
	char *line=(char *) malloc(65536);
	int section=0;
	char *buttons=NULL, *spaces=NULL;
	char *button=NULL, *space=NULL;
	char *btn=NULL, *spc=NULL;
	int num=1;
	in=fopen(QFile::encodeName(old), "r");
	if(!in) goto err;
	while(!feof(in)) {
		fgets(line, 65536, in);
		while(isspace(line[strlen(line)-1]))
			line[strlen(line)-1]=0;
		if(!strcasecmp(line, "[kpanelButtons]"))
			section=1;
		else if(line[0]=='[')
			section=0;
		else if(section && !strncasecmp(line, "Buttons=", 8))
			buttons=strdup(line+8);
		else if(section && !strncasecmp(line, "ButtonDelta=", 12))
			spaces=strdup(line+12);
	}
	fclose(in); in=NULL;
	if(!buttons) goto err;
	out=fopen(QFile::encodeName(nw), "w");
	if(!out) goto err;
	btn=(char *)malloc(strlen(buttons)*2+2);
	if(spaces)
		spc=(char *)malloc(strlen(spaces)*2+2);
	button=strtok_r(buttons, ",", &btn);
	if(spaces)
		space=strtok_r(spaces, ",", &spc);
	while(button) {
		if(strlen(button)) {
			if(strcasecmp(button, "system"))
				fprintf(out, "[URLButton_%u]\n", num++);
			else
				fprintf(out, "[KMenuButton_1]\n");
			if(space && strlen(space))
				fprintf(out, "FreeSpace=%s\n", space);
			else
				fprintf(out, "FreeSpace=0\n");
			if(strcasecmp(button, "system")) {
				char *loc;
				loc=replace(button, "$$KDEAPPS", "/usr/share/applnk");
				if(strstr(loc, "/usr/share") && strstr(loc, ".kdelnk")) {
					/* Let's just assume this is a standard tool... */
					char *l=replace(loc, ".kdelnk", ".desktop");
					free(loc);
					if(strstr(l, "kvt.desktop")) {
						loc=replace(l, "kvt.desktop", "konsole.desktop");
						free(l);
					} else {
						free(loc);
						loc=l;
					}
					
				}
				fprintf(out, "URL=%s\n", loc);
				free(loc);
			}
		}
		button=strtok_r(NULL, ",", &btn);
		if(space) space=strtok_r(NULL, ",", &spc);
	}
	fclose(out); out=NULL;
err:
	if(in) fclose(in);
	if(out) fclose(out);
	if(buttons) free(buttons);
	if(line) free(line);
	if(space) free(space);
	return 0;
}

#define i18n(x) gettext(x)

int main(int argc, char **argv)
{
	struct passwd *p;
	char *olddir, *newdir;
	char *fixme;

	setlocale(LC_ALL, "");
	bindtextdomain("desktopconv", "/usr/share/locale");
	textdomain("desktopconv");
	
	p=getpwuid(geteuid());
	if(p==NULL) {
		fprintf(stderr, i18n("Can't determine user!"));
		exit(1);
	}
	if(!QFile::exists(QString(p->pw_dir) + "/.kde/share/config/kpanelrc"))
		exit(0);

	QApplication app(argc, argv);
	int todo=QMessageBox::warning(0, i18n("Update configuration files"), i18n("Desktopconv has detected KDE 1.x config files in your home directory.\nDo you want to remove them (recommended), try to convert them, or ignore them?"), i18n("&Remove"), i18n("&Convert"), i18n("&Ignore"));
	if(todo==2) // Ignore
		exit(0);
	else if(todo==0) { // Remove
		system("/bin/rm -rf ~/.kde ~/Desktop ~/.kderc");
		exit(0);
	}

	// Convert...
	updateDesktop(QString(p->pw_dir) + "/Desktop", true);
	updateDesktop(QString(p->pw_dir) + "/.kde/share/applnk", false);
	updatePanel(p->pw_dir);

  // remove it after convert
  system("/bin/rm -f ~/.kde/share/config/kpanelrc");
	return 0;
}	
